# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Resource Policy State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure resource group exists:
            azurerm.resource.group.present:
                - name: my_rg
                - location: westus
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure resource group is absent:
            azurerm.resource.group.absent:
                - name: other_rg
                - connection_auth: {{ profile }}

'''

# Import Python libs
from __future__ import absolute_import
import json
import logging

# Import Salt libs
import salt.utils.files

log = logging.getLogger(__name__)


async def definition_present(hub, ctx, name, policy_rule=None, policy_type=None, mode=None, display_name=None, description=None,
                       metadata=None, parameters=None, policy_rule_json=None, policy_rule_file=None,
                       template='jinja', source_hash=None, source_hash_name=None, skip_verify=False,
                       connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a security policy definition exists.

    :param name:
        Name of the policy definition.

    :param policy_rule:
        A YAML dictionary defining the policy rule. See `Azure Policy Definition documentation
        <https://docs.microsoft.com/en-us/azure/azure-policy/policy-definition#policy-rule>`_ for details on the
        structure. One of ``policy_rule``, ``policy_rule_json``, or ``policy_rule_file`` is required, in that order of
        precedence for use if multiple parameters are used.

    :param policy_rule_json:
        A text field defining the entirety of a policy definition in JSON. See `Azure Policy Definition documentation
        <https://docs.microsoft.com/en-us/azure/azure-policy/policy-definition#policy-rule>`_ for details on the
        structure. One of ``policy_rule``, ``policy_rule_json``, or ``policy_rule_file`` is required, in that order of
        precedence for use if multiple parameters are used. Note that the `name` field in the JSON will override the
        ``name`` parameter in the state.

    :param policy_rule_file:
        The source of a JSON file defining the entirety of a policy definition. See `Azure Policy Definition
        documentation <https://docs.microsoft.com/en-us/azure/azure-policy/policy-definition#policy-rule>`_ for
        details on the structure. One of ``policy_rule``, ``policy_rule_json``, or ``policy_rule_file`` is required,
        in that order of precedence for use if multiple parameters are used. Note that the `name` field in the JSON
        will override the ``name`` parameter in the state.

    :param skip_verify:
        Used for the ``policy_rule_file`` parameter. If ``True``, hash verification of remote file sources
        (``http://``, ``https://``, ``ftp://``) will be skipped, and the ``source_hash`` argument will be ignored.

    :param source_hash:
        This can be a source hash string or the URI of a file that contains source hash strings.

    :param source_hash_name:
        When ``source_hash`` refers to a hash file, Salt will try to find the correct hash by matching the
        filename/URI associated with that hash.

    :param policy_type:
        The type of policy definition. Possible values are NotSpecified, BuiltIn, and Custom. Only used with the
        ``policy_rule`` parameter.

    :param mode:
        The policy definition mode. Possible values are NotSpecified, Indexed, and All. Only used with the
        ``policy_rule`` parameter.

    :param display_name:
        The display name of the policy definition. Only used with the ``policy_rule`` parameter.

    :param description:
        The policy definition description. Only used with the ``policy_rule`` parameter.

    :param metadata:
        The policy definition metadata defined as a dictionary. Only used with the ``policy_rule`` parameter.

    :param parameters:
        Required dictionary if a parameter is used in the policy rule. Only used with the ``policy_rule`` parameter.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure policy definition exists:
            azurerm.resource.policy.definition_present:
                - name: testpolicy
                - display_name: Test Policy
                - description: Test policy for testing policies.
                - policy_rule:
                    if:
                      allOf:
                        - equals: Microsoft.Compute/virtualMachines/write
                          source: action
                        - field: location
                          in:
                            - eastus
                            - eastus2
                            - centralus
                    then:
                      effect: deny
                - connection_auth: {{ profile }}

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    if not policy_rule and not policy_rule_json and not policy_rule_file:
        ret['comment'] = 'One of "policy_rule", "policy_rule_json", or "policy_rule_file" is required!'
        return ret

    if sum(x is not None for x in [policy_rule, policy_rule_json, policy_rule_file]) > 1:
        ret['comment'] = 'Only one of "policy_rule", "policy_rule_json", or "policy_rule_file" is allowed!'
        return ret

    if ((policy_rule_json or policy_rule_file) and
       (policy_type or mode or display_name or description or metadata or parameters)):
        ret['comment'] = 'Policy definitions cannot be passed when "policy_rule_json" or "policy_rule_file" is defined!'
        return ret

    temp_rule = {}
    if policy_rule_json:
        try:
            temp_rule = json.loads(policy_rule_json)
        except Exception as exc:
            ret['comment'] = 'Unable to load policy rule json! ({0})'.format(exc)
            return ret
    elif policy_rule_file:
        try:
            # pylint: disable=unused-variable
            sfn, source_sum, comment_ = __salt__['file.get_managed'](
                None,
                template,
                policy_rule_file,
                source_hash,
                source_hash_name,
                None,
                None,
                None,
                __env__,
                None,
                None,
                skip_verify=skip_verify,
                **kwargs
            )
        except Exception as exc:
            ret['comment'] = 'Unable to locate policy rule file "{0}"! ({1})'.format(policy_rule_file, exc)
            return ret

        if not sfn:
            ret['comment'] = 'Unable to locate policy rule file "{0}"!)'.format(policy_rule_file)
            return ret

        try:
            with salt.utils.files.fopen(sfn, 'r') as prf:
                temp_rule = json.load(prf)
        except Exception as exc:
            ret['comment'] = 'Unable to load policy rule file "{0}"! ({1})'.format(policy_rule_file, exc)
            return ret

        if sfn:
            salt.utils.files.remove(sfn)

    policy_name = name
    if policy_rule_json or policy_rule_file:
        if temp_rule.get('name'):
            policy_name = temp_rule.get('name')
        policy_rule = temp_rule.get('properties', {}).get('policyRule')
        policy_type = temp_rule.get('properties', {}).get('policyType')
        mode = temp_rule.get('properties', {}).get('mode')
        display_name = temp_rule.get('properties', {}).get('displayName')
        description = temp_rule.get('properties', {}).get('description')
        metadata = temp_rule.get('properties', {}).get('metadata')
        parameters = temp_rule.get('properties', {}).get('parameters')

    policy = await hub.exec.azurerm.resource.policy.definition_get(name, azurearm_log_level='info', **connection_auth)

    if 'error' not in policy:
        if policy_type and policy_type.lower() != policy.get('policy_type', '').lower():
            ret['changes']['policy_type'] = {
                'old': policy.get('policy_type'),
                'new': policy_type
            }

        if (mode or '').lower() != policy.get('mode', '').lower():
            ret['changes']['mode'] = {
                'old': policy.get('mode'),
                'new': mode
            }

        if (display_name or '').lower() != policy.get('display_name', '').lower():
            ret['changes']['display_name'] = {
                'old': policy.get('display_name'),
                'new': display_name
            }

        if (description or '').lower() != policy.get('description', '').lower():
            ret['changes']['description'] = {
                'old': policy.get('description'),
                'new': description
            }

        rule_changes = await hub.exec.utils.dictdiffer.deep_diff(policy.get('policy_rule', {}), policy_rule or {})
        if rule_changes:
            ret['changes']['policy_rule'] = rule_changes

        meta_changes = await hub.exec.utils.dictdiffer.deep_diff(policy.get('metadata', {}), metadata or {})
        if meta_changes:
            ret['changes']['metadata'] = meta_changes

        param_changes = await hub.exec.utils.dictdiffer.deep_diff(policy.get('parameters', {}), parameters or {})
        if param_changes:
            ret['changes']['parameters'] = param_changes

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Policy definition {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['comment'] = 'Policy definition {0} would be updated.'.format(name)
            ret['result'] = None
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': policy_name,
                'policy_type': policy_type,
                'mode': mode,
                'display_name': display_name,
                'description': description,
                'metadata': metadata,
                'parameters': parameters,
                'policy_rule': policy_rule,
            }
        }

    if ctx['test']:
        ret['comment'] = 'Policy definition {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    # Convert OrderedDict to dict
    if isinstance(metadata, dict):
        metadata = json.loads(json.dumps(metadata))
    if isinstance(parameters, dict):
        parameters = json.loads(json.dumps(parameters))

    policy_kwargs = kwargs.copy()
    policy_kwargs.update(connection_auth)

    policy = await hub.exec.azurerm.resource.policy.definition_create_or_update(
        name=policy_name,
        policy_rule=policy_rule,
        policy_type=policy_type,
        mode=mode,
        display_name=display_name,
        description=description,
        metadata=metadata,
        parameters=parameters,
        **policy_kwargs
    )

    if 'error' not in policy:
        ret['result'] = True
        ret['comment'] = 'Policy definition {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create policy definition {0}! ({1})'.format(name, policy.get('error'))
    return ret


async def definition_absent(hub, name, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a policy definition does not exist in the current subscription.

    :param name:
        Name of the policy definition.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    policy = await hub.exec.azurerm.resource.policy.definition_get(name, azurearm_log_level='info', **connection_auth)

    if 'error' in policy:
        ret['result'] = True
        ret['comment'] = 'Policy definition {0} is already absent.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Policy definition {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': policy,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.resource.policy.definition_delete(name, **connection_auth)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Policy definition {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': policy,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete policy definition {0}!'.format(name)
    return ret


async def assignment_present(hub, ctx, name, scope, definition_name, display_name=None, description=None, assignment_type=None,
                              parameters=None, connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a security policy assignment exists.

    :param name:
        Name of the policy assignment.

    :param scope:
        The scope of the policy assignment.

    :param definition_name:
        The name of the policy definition to assign.

    :param display_name:
        The display name of the policy assignment.

    :param description:
        The policy assignment description.

    :param assignment_type:
        The type of policy assignment.

    :param parameters:
        Required dictionary if a parameter is used in the policy rule.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure policy assignment exists:
            azurerm.resource.policy.assignment_present:
                - name: testassign
                - scope: /subscriptions/bc75htn-a0fhsi-349b-56gh-4fghti-f84852
                - definition_name: testpolicy
                - display_name: Test Assignment
                - description: Test assignment for testing assignments.
                - connection_auth: {{ profile }}

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    policy = await hub.exec.azurerm.resource.policy.assignment_get(
        name,
        scope,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in policy:
        if assignment_type and assignment_type.lower() != policy.get('type', '').lower():
            ret['changes']['type'] = {
                'old': policy.get('type'),
                'new': assignment_type
            }

        if scope.lower() != policy['scope'].lower():
            ret['changes']['scope'] = {
                'old': policy['scope'],
                'new': scope
            }

        pa_name = policy['policy_definition_id'].split('/')[-1]
        if definition_name.lower() != pa_name.lower():
            ret['changes']['definition_name'] = {
                'old': pa_name,
                'new': definition_name
            }

        if (display_name or '').lower() != policy.get('display_name', '').lower():
            ret['changes']['display_name'] = {
                'old': policy.get('display_name'),
                'new': display_name
            }

        if (description or '').lower() != policy.get('description', '').lower():
            ret['changes']['description'] = {
                'old': policy.get('description'),
                'new': description
            }

        param_changes = await hub.exec.utils.dictdiffer.deep_diff(policy.get('parameters', {}), parameters or {})
        if param_changes:
            ret['changes']['parameters'] = param_changes

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Policy assignment {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['comment'] = 'Policy assignment {0} would be updated.'.format(name)
            ret['result'] = None
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'scope': scope,
                'definition_name': definition_name,
                'type': assignment_type,
                'display_name': display_name,
                'description': description,
                'parameters': parameters,
            }
        }

    if ctx['test']:
        ret['comment'] = 'Policy assignment {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    if isinstance(parameters, dict):
        parameters = json.loads(json.dumps(parameters))

    policy_kwargs = kwargs.copy()
    policy_kwargs.update(connection_auth)
    policy = await hub.exec.azurerm.resource.policy.assignment_create(
        name=name,
        scope=scope,
        definition_name=definition_name,
        type=assignment_type,
        display_name=display_name,
        description=description,
        parameters=parameters,
        **policy_kwargs
    )

    if 'error' not in policy:
        ret['result'] = True
        ret['comment'] = 'Policy assignment {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create policy assignment {0}! ({1})'.format(name, policy.get('error'))
    return ret


async def assignment_absent(hub, ctx, name, scope, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a policy assignment does not exist in the provided scope.

    :param name:
        Name of the policy assignment.

    :param scope:
        The scope of the policy assignment.

    connection_auth
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    policy = await hub.exec.azurerm.resource.policy.assignment_get(
        name,
        scope,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in policy:
        ret['result'] = True
        ret['comment'] = 'Policy assignment {0} is already absent.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Policy assignment {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': policy,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.resource.policy.assignment_delete(name, scope, **connection_auth)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Policy assignment {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': policy,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete policy assignment {0}!'.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Compute Disk Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.compute.models  # pylint: disable=unused-import
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    from msrestazure.tools import is_valid_resource_id
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a disk.

    :param name: The disk to delete.

    :param resource_group: The resource group name assigned to the disk.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.disk.delete testdisk testgroup

    '''
    result = False
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        compconn.disks.delete(
            resource_group_name=resource_group,
            disk_name=name
        )
        result = True

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Resource Deployment Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 2.7.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.34.3
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.2
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
from json import loads, dumps
import logging

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.resource.resources.models  # pylint: disable=unused-import
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def operation_get(hub, operation, deployment, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get a deployment operation within a deployment.

    :param operation: The operation ID of the operation within the deployment.

    :param deployment: The name of the deployment containing the operation.

    :param resource_group: The resource group name assigned to the
        deployment.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_operation_get XXXXX testdeploy testgroup

    '''
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        operation = resconn.deployment_operations.get(
            resource_group_name=resource_group,
            deployment_name=deployment,
            operation_id=operation
        )

        result = operation.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def operations_list(hub, name, resource_group, result_limit=10, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all deployment operations within a deployment.

    :param name: The name of the deployment to query.

    :param resource_group: The resource group name assigned to the
        deployment.

    :param result_limit: (Default: 10) The limit on the list of deployment
        operations.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_operations_list testdeploy testgroup

    '''
    result = {}
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        operations = await hub.exec.utils.azurerm.paged_object_to_list(
            resconn.deployment_operations.list(
                resource_group_name=resource_group,
                deployment_name=name,
                top=result_limit
            )
        )

        for oper in operations:
            result[oper['operation_id']] = oper
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a deployment.

    :param name: The name of the deployment to delete.

    :param resource_group: The resource group name assigned to the
        deployment.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_delete testdeploy testgroup

    '''
    result = False
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        deploy = resconn.deployments.delete(
            deployment_name=name,
            resource_group_name=resource_group
        )
        deploy.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)

    return result


async def check_existence(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Check the existence of a deployment.

    :param name: The name of the deployment to query.

    :param resource_group: The resource group name assigned to the
        deployment.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_check_existence testdeploy testgroup

    '''
    result = False
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        result = resconn.deployments.check_existence(
            deployment_name=name,
            resource_group_name=resource_group
        )
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)

    return result


async def create_or_update(hub, name, resource_group, deploy_mode='incremental',
                                debug_setting='none', deploy_params=None,
                                parameters_link=None, deploy_template=None,
                                template_link=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deploys resources to a resource group.

    :param name: The name of the deployment to create or update.

    :param resource_group: The resource group name assigned to the
        deployment.

    :param deploy_mode: The mode that is used to deploy resources. This value can be either
        'incremental' or 'complete'. In Incremental mode, resources are deployed without deleting
        existing resources that are not included in the template. In Complete mode, resources
        are deployed and existing resources in the resource group that are not included in
        the template are deleted. Be careful when using Complete mode as you may
        unintentionally delete resources.

    :param debug_setting: The debug setting of the deployment. The permitted values are 'none',
        'requestContent', 'responseContent', or 'requestContent,responseContent'. By logging
        information about the request or response, you could potentially expose sensitive data
        that is retrieved through the deployment operations.

    :param deploy_params: JSON string containing name and value pairs that define the deployment
        parameters for the template. You use this element when you want to provide the parameter
        values directly in the request rather than link to an existing parameter file. Use either
        the parameters_link property or the deploy_params property, but not both.

    :param parameters_link: The URI of a parameters file. You use this element to link to an existing
        parameters file. Use either the parameters_link property or the deploy_params property, but not both.

    :param deploy_template: JSON string of template content. You use this element when you want to pass
        the template syntax directly in the request rather than link to an existing template. Use either
        the template_link property or the deploy_template property, but not both.

    :param template_link: The URI of the template. Use either the template_link property or the
        deploy_template property, but not both.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_create_or_update testdeploy testgroup

    '''
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)

    prop_kwargs = {'mode': deploy_mode}
    prop_kwargs['debug_setting'] = {'detail_level': debug_setting}

    if deploy_params:
        prop_kwargs['parameters'] = deploy_params
    else:
        if isinstance(parameters_link, dict):
            prop_kwargs['parameters_link'] = parameters_link
        else:
            prop_kwargs['parameters_link'] = {'uri': parameters_link}

    if deploy_template:
        prop_kwargs['template'] = deploy_template
    else:
        if isinstance(template_link, dict):
            prop_kwargs['template_link'] = template_link
        else:
            prop_kwargs['template_link'] = {'uri': template_link}

    deploy_kwargs = kwargs.copy()
    deploy_kwargs.update(prop_kwargs)

    try:
        deploy_model = await hub.exec.utils.azurerm.create_object_model(
            'resource',
            'DeploymentProperties',
            **deploy_kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        validate = deployment_validate(
            name=name,
            resource_group=resource_group,
            **deploy_kwargs
        )
        if 'error' in validate:
            result = validate
        else:
            deploy = resconn.deployments.create_or_update(
                deployment_name=name,
                resource_group_name=resource_group,
                properties=deploy_model
            )
            deploy.wait()
            deploy_result = deploy.result()
            result = deploy_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get details about a specific deployment.

    :param name: The name of the deployment to query.

    :param resource_group: The resource group name assigned to the
        deployment.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_get testdeploy testgroup

    '''
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        deploy = resconn.deployments.get(
            deployment_name=name,
            resource_group_name=resource_group
        )
        result = deploy.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def cancel(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Cancel a deployment if in 'Accepted' or 'Running' state.

    :param name: The name of the deployment to cancel.

    :param resource_group: The resource group name assigned to the
        deployment.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_cancel testdeploy testgroup

    '''
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        resconn.deployments.cancel(
            deployment_name=name,
            resource_group_name=resource_group
        )
        result = {'result': True}
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {
            'error': str(exc),
            'result': False
        }

    return result


async def validate(hub, name, resource_group, deploy_mode=None,
                        debug_setting=None, deploy_params=None,
                        parameters_link=None, deploy_template=None,
                        template_link=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Validates whether the specified template is syntactically correct
    and will be accepted by Azure Resource Manager.

    :param name: The name of the deployment to validate.

    :param resource_group: The resource group name assigned to the
        deployment.

    :param deploy_mode: The mode that is used to deploy resources. This value can be either
        'incremental' or 'complete'. In Incremental mode, resources are deployed without deleting
        existing resources that are not included in the template. In Complete mode, resources
        are deployed and existing resources in the resource group that are not included in
        the template are deleted. Be careful when using Complete mode as you may
        unintentionally delete resources.

    :param debug_setting: The debug setting of the deployment. The permitted values are 'none',
        'requestContent', 'responseContent', or 'requestContent,responseContent'. By logging
        information about the request or response, you could potentially expose sensitive data
        that is retrieved through the deployment operations.

    :param deploy_params: JSON string containing name and value pairs that define the deployment
        parameters for the template. You use this element when you want to provide the parameter
        values directly in the request rather than link to an existing parameter file. Use either
        the parameters_link property or the deploy_params property, but not both.

    :param parameters_link: The URI of a parameters file. You use this element to link to an existing
        parameters file. Use either the parameters_link property or the deploy_params property, but not both.

    :param deploy_template: JSON string of template content. You use this element when you want to pass
        the template syntax directly in the request rather than link to an existing template. Use either
        the template_link property or the deploy_template property, but not both.

    :param template_link: The URI of the template. Use either the template_link property or the
        deploy_template property, but not both.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_validate testdeploy testgroup

    '''
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)

    prop_kwargs = {'mode': deploy_mode}
    prop_kwargs['debug_setting'] = {'detail_level': debug_setting}

    if deploy_params:
        prop_kwargs['parameters'] = deploy_params
    else:
        if isinstance(parameters_link, dict):
            prop_kwargs['parameters_link'] = parameters_link
        else:
            prop_kwargs['parameters_link'] = {'uri': parameters_link}

    if deploy_template:
        prop_kwargs['template'] = deploy_template
    else:
        if isinstance(template_link, dict):
            prop_kwargs['template_link'] = template_link
        else:
            prop_kwargs['template_link'] = {'uri': template_link}

    deploy_kwargs = kwargs.copy()
    deploy_kwargs.update(prop_kwargs)

    try:
        deploy_model = await hub.exec.utils.azurerm.create_object_model(
            'resource',
            'DeploymentProperties',
            **deploy_kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        local_validation = deploy_model.validate()
        if local_validation:
            raise local_validation[0]

        deploy = resconn.deployments.validate(
            deployment_name=name,
            resource_group_name=resource_group,
            properties=deploy_model
        )
        result = deploy.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def export_template(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Exports the template used for the specified deployment.

    :param name: The name of the deployment to query.

    :param resource_group: The resource group name assigned to the
        deployment.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployment_export_template testdeploy testgroup

    '''
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        deploy = resconn.deployments.export_template(
            deployment_name=name,
            resource_group_name=resource_group
        )
        result = deploy.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all deployments within a resource group.

    CLI Example:

    .. code-block:: bash

        azurearm_resource.deployments_list testgroup

    '''
    result = {}
    resconn = await hub.exec.utils.azurerm.get_client('resource', **kwargs)
    try:
        deployments = await hub.exec.utils.azurerm.paged_object_to_list(
            resconn.deployments.list_by_resource_group(
                resource_group_name=resource_group
            )
        )

        for deploy in deployments:
            result[deploy['name']] = deploy
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('resource', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Compute Virtual Machine State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure virtual machine exists:
          azurerm.compute.virtual_machine.present:
            - name: idem-vm01
            - resource_group: idem
            - vm_size: Standard_B1s
            - virtual_network: vnet1
            - subnet: default
            - allocate_public_ip: True
            - ssh_public_keys:
                - /home/myuser/.ssh/id_rsa.pub
            - tags:
                contact_name: Elmer Fudd Gantry
            - connection_auth: {{ profile }}

        Ensure virtual machine is absent:
            azurerm.compute.virtual_machine.absent:
                - name: idem-vm01
                - resource_group: idem
                - connection_auth: {{ profile }}

'''

# Python libs
from __future__ import absolute_import
import logging

log = logging.getLogger(__name__)


async def present(hub, ctx, name, resource_group, tags=None, connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a virtual machine exists.

    :param name:
        Name of the virtual machine.

    :param resource_group:
        The resource group assigned to the virtual machine.

    :param tags:
        A dictionary of strings can be passed as tag metadata to the virtual machine object.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure virtual machine exists:
            azurerm.compute.virtual_machine.present:
                - name: idem-vm01
                - resource_group: idem
                - vm_size: Standard_B1s
                - virtual_network: vnet1
                - subnet: default
                - allocate_public_ip: True
                - ssh_public_keys:
                    - /home/myuser/.ssh/id_rsa.pub
                - tags:
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    vm = await hub.exec.azurerm.compute.virtual_machine.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    new_vm = True

    if 'error' not in vm:
        new_vm = False

        tag_changes = await hub.exec.utils.dictdiffer.deep_diff(vm.get('tags', {}), tags or {})

        if tag_changes:
            ret['changes']['tags'] = tag_changes

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Virtual machine {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Virtual machine {0} would be updated.'.format(name)
            return ret

    if ctx['test']:
        ret['comment'] = 'Virtual machine {0} would be created.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'tags': tags,
                **kwargs
            }
        }
        return ret

    vm_kwargs = kwargs.copy()
    vm_kwargs.update(connection_auth)

    vm = await hub.exec.azurerm.compute.virtual_machine.create_or_update(
        name=name,
        resource_group=resource_group,
        tags=tags,
        **vm_kwargs
    )

    if new_vm:
        ret['changes'] = {
            'old': {},
            'new': vm
        }

    if 'error' not in vm:
        ret['result'] = True
        ret['comment'] = 'Virtual machine {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create virtual machine {0}! ({1})'.format(name, vm.get('error'))
    return ret


async def absent(hub, ctx, name, resource_group, connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a virtual machine does not exist in a resource group.

    :param name:
        Name of the virtual machine.

    :param resource_group:
        Name of the resource group containing the virtual machine.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    vm = await hub.exec.azurerm.compute.virtual_machine.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in vm:
        ret['result'] = True
        ret['comment'] = 'Virtual machine {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Virtual machine {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': vm,
            'new': {},
        }
        return ret

    vm_kwargs = kwargs.copy()
    vm_kwargs.update(connection_auth)

    deleted = await hub.exec.azurerm.compute.virtual_machine.delete(name, resource_group, **vm_kwargs)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Virtual machine {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': vm,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete virtual machine {0}!'.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Network Route State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure virtual network exists:
            azurerm.network.virtual_network.present:
                - name: my_vnet
                - resource_group: my_rg
                - address_prefixes:
                    - '10.0.0.0/8'
                    - '192.168.0.0/16'
                - dns_servers:
                    - '8.8.8.8'
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure virtual network is absent:
            azurerm.network.virtual_network.absent:
                - name: other_vnet
                - resource_group: my_rg
                - connection_auth: {{ profile }}

'''

# Python libs
from __future__ import absolute_import
import logging
import re

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

log = logging.getLogger(__name__)


async def table_present(hub, ctx, name, resource_group, tags=None, routes=None, disable_bgp_route_propagation=None,
                  connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a route table exists.

    :param name:
        Name of the route table.

    :param resource_group:
        The resource group assigned to the route table.

    :param routes:
        An optional list of dictionaries representing valid Route objects contained within a route table. See the
        documentation for the route_present state or route_create_or_update execution module for more information on
        required and optional parameters for routes. The routes are only managed if this parameter is present. When this
        parameter is absent, implemented routes will not be removed, and will merely become unmanaged.

    :param disable_bgp_route_propagation:
        An optional boolean parameter setting whether to disable the routes learned by BGP on the route table.

    :param tags:
        A dictionary of strings can be passed as tag metadata to the route table object.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure route table exists:
            azurerm.network.route.table_present:
                - name: rt1
                - resource_group: group1
                - routes:
                  - name: rt1_route1
                    address_prefix: '0.0.0.0/0'
                    next_hop_type: internet
                  - name: rt1_route2
                    address_prefix: '192.168.0.0/16'
                    next_hop_type: vnetlocal
                - tags:
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}
                - require:
                  - azurearm_resource: Ensure resource group exists

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    rt_tbl = await hub.exec.azurerm.network.route.table_get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in rt_tbl:
        # tag changes
        tag_changes = await hub.exec.utils.dictdiffer.deep_diff(rt_tbl.get('tags', {}), tags or {})
        if tag_changes:
            ret['changes']['tags'] = tag_changes

        # disable_bgp_route_propagation changes
        # pylint: disable=line-too-long
        if disable_bgp_route_propagation and (disable_bgp_route_propagation != rt_tbl.get('disable_bgp_route_propagation')):
            ret['changes']['disable_bgp_route_propagation'] = {
                'old': rt_tbl.get('disable_bgp_route_propagation'),
                'new': disable_bgp_route_propagation
            }

        # routes changes
        if routes:
            comp_ret = await hub.exec.utils.azurerm.compare_list_of_dicts(rt_tbl.get('routes', []), routes)

            if comp_ret.get('comment'):
                ret['comment'] = '"routes" {0}'.format(comp_ret['comment'])
                return ret

            if comp_ret.get('changes'):
                ret['changes']['routes'] = comp_ret['changes']

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Route table {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Route table {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'tags': tags,
                'routes': routes,
                'disable_bgp_route_propagation': disable_bgp_route_propagation,
            }
        }

    if ctx['test']:
        ret['comment'] = 'Route table {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    rt_tbl_kwargs = kwargs.copy()
    rt_tbl_kwargs.update(connection_auth)

    rt_tbl = await hub.exec.azurerm.network.route.table_create_or_update(
        name=name,
        resource_group=resource_group,
        disable_bgp_route_propagation=disable_bgp_route_propagation,
        routes=routes,
        tags=tags,
        **rt_tbl_kwargs
    )

    if 'error' not in rt_tbl:
        ret['result'] = True
        ret['comment'] = 'Route table {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create route table {0}! ({1})'.format(name, rt_tbl.get('error'))
    return ret


async def table_absent(hub, ctx, name, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a route table does not exist in the resource group.

    :param name:
        Name of the route table.

    :param resource_group:
        The resource group assigned to the route table.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    rt_tbl = await hub.exec.azurerm.network.route.table_get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in rt_tbl:
        ret['result'] = True
        ret['comment'] = 'Route table {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Route table {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': rt_tbl,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.network.route.table_delete(name, resource_group, **connection_auth)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Route table {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': rt_tbl,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete route table {0}!'.format(name)
    return ret


async def present(hub, ctx, name, address_prefix, next_hop_type, route_table, resource_group, next_hop_ip_address=None,
            connection_auth=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a route exists within a route table.

    :param name:
        Name of the route.

    :param address_prefix:
        The destination CIDR to which the route applies.

    :param next_hop_type:
        The type of Azure hop the packet should be sent to. Possible values are: 'VirtualNetworkGateway', 'VnetLocal',
        'Internet', 'VirtualAppliance', and 'None'.

    :param next_hop_ip_address:
        The IP address packets should be forwarded to. Next hop values are only allowed in routes where the next hop
        type is 'VirtualAppliance'.

    :param route_table:
        The name of the existing route table which will contain the route.

    :param resource_group:
        The resource group assigned to the route table.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure route exists:
            azurerm.network.route.present:
                - name: rt1_route2
                - route_table: rt1
                - resource_group: group1
                - address_prefix: '192.168.0.0/16'
                - next_hop_type: vnetlocal
                - connection_auth: {{ profile }}
                - require:
                  - azurearm_network: Ensure route table exists

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    route = await hub.exec.azurerm.network.route.get(
        name,
        route_table,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in route:
        if address_prefix != route.get('address_prefix'):
            ret['changes']['address_prefix'] = {
                'old': route.get('address_prefix'),
                'new': address_prefix
            }

        if next_hop_type.lower() != route.get('next_hop_type', '').lower():
            ret['changes']['next_hop_type'] = {
                'old': route.get('next_hop_type'),
                'new': next_hop_type
            }

        if next_hop_type.lower() == 'virtualappliance' and next_hop_ip_address != route.get('next_hop_ip_address'):
            ret['changes']['next_hop_ip_address'] = {
                'old': route.get('next_hop_ip_address'),
                'new': next_hop_ip_address
            }

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Route {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Route {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'address_prefix': address_prefix,
                'next_hop_type': next_hop_type,
                'next_hop_ip_address': next_hop_ip_address
            }
        }

    if ctx['test']:
        ret['comment'] = 'Route {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    route_kwargs = kwargs.copy()
    route_kwargs.update(connection_auth)

    route = await hub.exec.azurerm.network.route.create_or_update(
        name=name,
        route_table=route_table,
        resource_group=resource_group,
        address_prefix=address_prefix,
        next_hop_type=next_hop_type,
        next_hop_ip_address=next_hop_ip_address,
        **route_kwargs
    )

    if 'error' not in route:
        ret['result'] = True
        ret['comment'] = 'Route {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create route {0}! ({1})'.format(name, route.get('error'))
    return ret


async def absent(hub, ctx, name, route_table, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a route table does not exist in the resource group.

    :param name:
        Name of the route table.

    :param route_table:
        The name of the existing route table containing the route.

    :param resource_group:
        The resource group assigned to the route table.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    route = await hub.exec.azurerm.network.route.get(
        name,
        route_table,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in route:
        ret['result'] = True
        ret['comment'] = 'Route {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Route {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': route,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.network.route.delete(name, route_table, resource_group, **connection_auth)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Route {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': route,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete route {0}!'.format(name)
    return ret



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Import python libs
import os
import sys
import shutil
from setuptools import setup, Command

PYPI_NAME = 'idem_azurerm'
NAME = 'idem_provider_azurerm'
DESC = ('Idem language provider for Azure')

# Version info -- read without importing
_locals = {}
with open('{}/version.py'.format(NAME)) as fp:
    exec(fp.read(), None, _locals)
VERSION = _locals['version']
SETUP_DIRNAME = os.path.dirname(__file__)
if not SETUP_DIRNAME:
    SETUP_DIRNAME = os.getcwd()


class Clean(Command):
    user_options = []
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        for subdir in (NAME, 'tests'):
            for root, dirs, files in os.walk(os.path.join(os.path.dirname(__file__), subdir)):
                for dir_ in dirs:
                    if dir_ == '__pycache__':
                        shutil.rmtree(os.path.join(root, dir_))


def discover_packages():
    modules = []
    for package in (NAME, ):
        for root, _, files in os.walk(os.path.join(SETUP_DIRNAME, package)):
            pdir = os.path.relpath(root, SETUP_DIRNAME)
            modname = pdir.replace(os.sep, '.')
            modules.append(modname)
    return modules


setup(name=PYPI_NAME,
      author='Nicholas Hughes',
      author_email='',
      url='',
      version=VERSION,
      description=DESC,
      classifiers=[
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Development Status :: 5 - Production/Stable',
          ],
      packages=discover_packages(),
      cmdclass={'clean': Clean},
      )



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Virtual Network Gateway Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.network.models  # pylint: disable=unused-import
    from msrestazure.tools import is_valid_resource_id, parse_resource_id
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def connection_create_or_update(hub, name, resource_group, virtual_network_gateway, connection_type, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Creates or updates a virtual network gateway connection in the specified resource group.

    :param name: The name of the virtual network gateway connection to create or update.

    :param resource_group: The name of the resource group.

    :param virtual_network_gateway: The name of the virtual network gateway that will
        be the first endpoint of the connection. This is immutable once set.

    :param connection_type: Gateway connection type. Possible values include:
        'IPsec', 'Vnet2Vnet', and 'ExpressRoute'. This is immutable once set.

    A second endpoint must be passed as a keyword argument:
      - For a connection of type 'Vnet2Vnet' a valid Resource ID representing a VirtualNetworkGateway Object must be
        passed as the virtual_network_gateway2 kwarg.
      - For a connection of type 'IPSec' a valid Resource ID representing a LocalNetworkGateway Object must be passed
        as the local_network_gateway2 kwarg.
      - For a connection of type 'ExpressRoute' a valid Resource ID representing an ExpressRouteCircuit Object must be
        passed as the peer kwarg.
    The second endpoint is immutable once set.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.connection_create_or_update test_name test_group
                  test_vnet_gw test_connection_type

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    # Converts the Resource ID of virtual_network_gateway into a VirtualNetworkGateway Object.
    # This endpoint will be where the connection originates.
    if not is_valid_resource_id(virtual_network_gateway):
        log.error(
            'Invalid Resource ID was specified as virtual_network_gateway!'
        )
        return False

    vnetgw1_parsed_id = parse_resource_id(virtual_network_gateway)

    try:
        vnetgw1_rg = vnetgw1_parsed_id['resource_group']
        vnetgw1_name = vnetgw1_parsed_id['resource_name']
    except KeyError as esc:
        log.error(
            'Invalid Resource ID was specified as virtual_network_gateway! (%s)',
            esc
        )
        return False

    vnetgw1 = virtual_network_gateway_get(
        name=vnetgw1_name,
        resource_group=vnetgw1_rg,
        **kwargs
    )

    if 'error' in vnetgw1:
        log.error(
            'Unable to find the resource specified in virtual_network_gateway!'
        )
        return False
    else:
        virtual_network_gateway = {'id': str(vnetgw1['id'])}

    # Check the Resource ID path of virtual_network_gateway2
    # We can't guarantee the validity of the object, so we hope you have the path correct...
    if kwargs.get('virtual_network_gateway2') and not is_valid_resource_id(kwargs['virtual_network_gateway2']):
        log.error(
            'Invalid Resource ID was specified as virtual_network_gateway2!'
        )
        return False

    # Check the Resource ID path of local_network_gateway2
    # We can't guarantee the validity of the object, so we hope you have the path correct...
    if kwargs.get('local_network_gateway2') and not is_valid_resource_id(kwargs['local_network_gateway2']):
        log.error(
            'Invalid Resource ID was specified as local_network_gateway2!'
        )
        return False

    if kwargs.get('virtual_network_gateway2'):
        kwargs['virtual_network_gateway2'] = {'id': kwargs.get('virtual_network_gateway2')}

    if kwargs.get('local_network_gateway2'):
        kwargs['local_network_gateway2'] = {'id': kwargs.get('local_network_gateway2')}

    try:
        connectionmodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'VirtualNetworkGatewayConnection',
            virtual_network_gateway1=virtual_network_gateway,
            connection_type=connection_type,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        connection = netconn.virtual_network_gateway_connections.create_or_update(
            resource_group_name=resource_group,
            virtual_network_gateway_connection_name=name,
            parameters=connectionmodel
        )
        connection.wait()
        connection_result = connection.result()
        result = connection_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def connection_get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets the details of a specified virtual network gateway connection.

    :param name: The name of the virtual network gateway connection to query.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.connection_get test_name test_group

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        connection = netconn.virtual_network_gateway_connections.get(
            resource_group_name=resource_group,
            virtual_network_gateway_connection_name=name
        )

        result = connection.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def connection_delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deletes the specified virtual network gateway connection.

    :param name: The name of the virtual network gateway connection that will be deleted.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.connection_delete test_name test_group

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        connection = netconn.virtual_network_gateway_connections.delete(
            resource_group_name=resource_group,
            virtual_network_gateway_connection_name=name
        )
        connection.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)

    return result


async def connection_set_shared_key(hub, name, resource_group, value, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Sets the shared key for a virtual network gateway connection object.

    :param name: The virtual network gateway connection name.

    :param resource_group: The name of the resource group.

    :param value: The new virtual network connection shared key value.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.connection_set_shared_key test_name \
                  test_group test_value

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    try:
        key = netconn.virtual_network_gateway_connections.set_shared_key(
            resource_group_name=resource_group,
            virtual_network_gateway_connection_name=name,
            value=value
        )

        key.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def connection_get_shared_key(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets information about the specified virtual network gateway connection shared key
        through the Network resource provider.

    :param name: The virtual network gateway connection shared key name.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.connection_get_shared_key test_name test_group

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        key = netconn.virtual_network_gateway_connections.get_shared_key(
            resource_group_name=resource_group,
            virtual_network_gateway_connection_name=name
        )

        result = key.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def connection_reset_shared_key(hub, name, resource_group, key_length=128, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Resets the virtual network gateway connection shared key for passed virtual network
        gateway connection in the specified resource group through Network resource provider.

    :param name: The name of the virtual network gateway connection that will have its shared key reset.

    :param resource_group: The name of the resource group.

    :param key_length: The virtual network connection reset shared key length, should between 1 and 128.
        Defaults to 128.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.connection_set_shared_key test_name \
                  test_group test_key_length

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        rkey = netconn.virtual_network_gateway_connections.reset_shared_key(
            resource_group_name=resource_group,
            virtual_network_gateway_connection_name=name,
            key_length=key_length
        )

        rkey.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def connections_list(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Lists all the virtual network gateway connections within a specified resource group.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.connections_list test_group

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        connections = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.virtual_network_gateway_connections.list(
                resource_group_name=resource_group
            )
        )

        for connection in connections:
            result[connection['name']] = connection
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Lists all virtual network gateways within a resource group.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.list test_group

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        gateways = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.virtual_network_gateways.list(
                resource_group_name=resource_group
            )
        )

        for gateway in gateways:
            result[gateway['name']] = gateway
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def create_or_update(hub, name, resource_group, virtual_network, ip_configurations, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Creates or updates a virtual network gateway in the specified resource group.

    :param name: The name of the virtual network gateway to be created or updated.

    :param resource_group: The name of the resource group.

    :param virtual_network: The name of the virtual network associated with the virtual network gateway.

    :param ip_configurations:
        A list of dictionaries representing valid VirtualNetworkGatewayIPConfiguration objects. Valid parameters are:
          - ``name``: The name of the resource that is unique within a resource group.
          - ``public_ip_address``: Name of an existing public IP address that'll be assigned to the IP config object.
          - ``private_ip_allocation_method``: The private IP allocation method. Possible values are:
                                              'Static' and 'Dynamic'.
          - ``subnet``: Name of an existing subnet inside of which the IP config will reside.
        If the active_active keyword argument is disabled, only one IP configuration dictionary is permitted.
        If the active_active keyword argument is enabled, two IP configuration dictionaries are required.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.create_or_update test_name test_group \
                  test_vnet test_ip_configs

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    # Loop through IP Configurations and build each dictionary to pass to model creation.
    if isinstance(ip_configurations, list):
        subnet = subnet_get(
            name='GatewaySubnet',
            virtual_network=virtual_network,
            resource_group=resource_group,
            **kwargs
        )
        if 'error' not in subnet:
            subnet = {'id': str(subnet['id'])}
            for ipconfig in ip_configurations:
                if 'name' in ipconfig:
                    ipconfig['subnet'] = subnet
                    if ipconfig.get('public_ip_address'):
                        pub_ip = public_ip_address_get(
                            name=ipconfig['public_ip_address'],
                            resource_group=resource_group,
                            **kwargs
                        )
                        if 'error' not in pub_ip:
                            ipconfig['public_ip_address'] = {'id': str(pub_ip['id'])}

    try:
        gatewaymodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'VirtualNetworkGateway',
            ip_configurations=ip_configurations,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        gateway = netconn.virtual_network_gateways.create_or_update(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name,
            parameters=gatewaymodel
        )
        gateway.wait()
        gateway_result = gateway.result()
        result = gateway_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets the details of a specific virtual network gateway within a specified resource group.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.get test_name test_group

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        gateway = netconn.virtual_network_gateways.get(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name
        )

        result = gateway.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Deletes the specified virtual network gateway.

    :param name: The name of the virtual network gateway that will be deleted.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.delete test_name test_group

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        gateway = netconn.virtual_network_gateways.delete(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name
        )
        gateway.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)

    return result


async def list_connections(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Lists all connections associated with a virtual network gateway.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.list_connections test_name test_group

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        connections = await hub.exec.utils.azurerm.paged_object_to_list(
            netconn.virtual_network_gateways.list_connections(
                resource_group_name=resource_group,
                virtual_network_gateway_name=name
            )
        )
        for connection in connections:
            result[connection['name']] = connection
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def reset(hub, name, resource_group, gateway_vip=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Resets the virtual network gateway in the specified resource group.

    :param name: The name of the virtual network gateway to reset.

    :param resource_group: The name of the resource group.

    :param gateway_vip: Virtual network gateway vip address supplied to the begin
        reset of the active-active feature enabled gateway.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.reset test_name test_group

    '''
    result = False
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        reset = netconn.virtual_network_gateways.reset(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name,
            gateway_vip=gateway_vip
        )
        reset.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def reset_vpn_client_shared_key(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Resets the VPN client shared key of the virtual network gateway in the specified resource group.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.reset_vpn_client_shared_key test_name test_group

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        reset = netconn.virtual_network_gateways.reset_vpn_client_shared_key(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name
        )

        reset_result = reset.result()
        result = reset_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def generatevpnclientpackage(hub, name, resource_group, processor_architecture, authentication_method,
                                   radius_server_auth_certificate=None, client_root_certificates=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Generates VPN client package for P2S client of the virtual network
        gateway in the specified resource group.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    :param processor_architecture: VPN client Processor Architecture. Possible values include: 'Amd64', 'X86'

    :param authentication_method: VPN client authentication method. Possible values include: 'EAPTLS', 'EAPMSCHAPv2'

    :param radius_server_auth_certificate: The public certificate data for the radius server authentication
        certificate as a Base-64 encoded string. Required only if external radius authentication has been configured
        with EAPTLS authentication.

    :param client_root_certificates: A list of client root certificates public certificate data encoded as Base-64
        strings. Optional parameter for external radius based authentication with EAPTLS.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.generatevpnclientpackage test_name test_group test_params

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    try:
        pkgmodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'VpnClientParameters',
            processor_architecture=processor_architecture,
            authentication_method=authentication_method,
            radius_server_auth_certificate=radius_server_auth_certificate,
            client_root_certificates=client_root_certificates
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        pkg = netconn.virtual_network_gateways.generatevpnclientpackage(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name,
            parameters=pkgmodel,
            **kwargs
        )

        result = pkg
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def generate_vpn_profile(hub, name, resource_group, processor_architecture, authentication_method,
                               radius_server_auth_certificate=None, client_root_certificates=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Generates VPN profile for P2S client of the virtual network gateway in the
        specified resource group. Used for IKEV2 and radius based authentication.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    :param processor_architecture: VPN client Processor Architecture. Possible values include: 'Amd64', 'X86'

    :param authentication_method: VPN client authentication method. Possible values include: 'EAPTLS', 'EAPMSCHAPv2'

    :param radius_server_auth_certificate: The public certificate data for the radius server authentication
        certificate as a Base-64 encoded string. Required only if external radius authentication has been configured
        with EAPTLS authentication.

    :param client_root_certificates: A list of client root certificates public certificate data encoded as Base-64
                                     strings. Optional parameter for external radius based authentication with EAPTLS.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.generate_vpn_profile test_name test_group test_params

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    try:
        profilemodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'VpnClientParameters',
            processor_architecture=processor_architecture,
            authentication_method=authentication_method,
            radius_server_auth_certificate=radius_server_auth_certificate,
            client_root_certificates=client_root_certificates
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        profile = netconn.virtual_network_gateways.generate_vpn_profile(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name,
            parameters=profilemodel,
            **kwargs
        )

        result = profile
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def get_vpn_profile_package_url(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets pre-generated VPN profile for P2S client of the virtual network gateway in the
        specified resource group.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.get_vpn_profile_package_url test_name test_group

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        url = netconn.virtual_network_gateways.get_vpn_profile_package_url(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name
        )

        result = url.result()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_bgp_peer_status(hub, name, resource_group, peer=None, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets the status of all BGP peers.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    :param peer: The IP address of the peer to retrieve the status of.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.get_bgp_peer_status test_name test_group test_peer

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        peers = netconn.virtual_network_gateways.get_bgp_peer_status(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name,
            peer=peer
        )

        peers_result = peers.result().as_dict()
        for bgp_peer in peers_result['value']:
            result['BGP peer'] = bgp_peer
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def supported_vpn_devices(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets a xml format representation for supported vpn devices.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.supported_vpn_devices test_name test_group

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        devices = netconn.virtual_network_gateways.supported_vpn_devices(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name
        )

        result = devices
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_learned_routes(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets a list of routes that the virtual network gateway has learned, including routes learned from BGP peers.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.get_learned_routes test_name test_group

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        routes = netconn.virtual_network_gateways.get_learned_routes(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name
        )

        routes_result = routes.result().as_dict()
        for route in routes_result['value']:
            result['route_list'] = route
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_advertised_routes(hub, name, resource_group, peer, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets a list of routes the virtual network gateway is advertising to a specified peer.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    :param peer: The IP address of the peer.

    CLI Example:
    .. code-block:: bash

        azurerm.network.virtual_network_gateway.get_learned_routes test_name test_group test_peer

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        routes = netconn.virtual_network_gateways.get_advertised_routes(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name,
            peer=peer
        )

        routes_result = routes.result().as_dict()
        for route in routes_result['value']:
            result['route_list'] = route
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def set_vpnclient_ipsec_parameters(hub, name, resource_group, sa_life_time_seconds, sa_data_size_kilobytes,
                                         ipsec_encryption, ipsec_integrity, ike_encryption, ike_integrity, dh_group,
                                         pfs_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Sets the vpnclient ipsec policy for P2S client of virtual network gateway in the
        specified resource group through Network resource provider.

    :param name: The name of the virtual network gateway.

    :param resource_group: The name of the resource group.

    The following parameters are for creating a VpnClientIPsecParameters object:

    :param sa_life_time_seconds: The IPSec Security Association (also called Quick Mode or Phase 2 SA)
        lifetime in seconds for P2S client. Must be between 300 - 172799 seconds.

    :param sa_data_size_kilobytes: The IPSec Security Association (also called Quick Mode or Phase 2 SA)
        payload size in KB for P2S client. Must be between 1024 - 2147483647 kilobytes.

    :param ipsec_encryption: The IPSec encryption algorithm (IKE phase 1). Possible values include:
        'None', 'DES', 'DES3', 'AES128', 'AES192', 'AES256', 'GCMAES128', 'GCMAES192', 'GCMAES256'

    :param ipsec_integrity: The IPSec integrity algorithm (IKE phase 1). Possible values include:
        'MD5', 'SHA1', 'SHA256', 'GCMAES128', 'GCMAES192', 'GCMAES256'

    :param ike_encryption: The IKE encryption algorithm (IKE phase 2). Possible values include:
        'DES', 'DES3', 'AES128', 'AES192', 'AES256', 'GCMAES256', 'GCMAES128'

    :param ike_integrity: The IKE integrity algorithm (IKE phase 2). Possible values include:
        'MD5', 'SHA1', 'SHA256', 'SHA384', 'GCMAES256', 'GCMAES128'

    :param dh_group: The DH Group used in IKE Phase 1 for initial SA. Possible values include:
        'None', 'DHGroup1', 'DHGroup2', 'DHGroup14', 'DHGroup2048', 'ECP256', 'ECP384', 'DHGroup24'

    :param pfs_group: The Pfs Group used in IKE Phase 2 for new child SA. Possible values include:
        'None', 'PFS1', 'PFS2', 'PFS2048', 'ECP256', 'ECP384', 'PFS24', 'PFS14', 'PFSMM'

    CLI Example:

    .. code-block:: bash

        azurerm.network.virtual_network_gateway.set_vpnclient_ipsec_parameters \
                  test_name test_group test_vpnclient_ipsec_params

    '''
    result = {}
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        paramsmodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'VpnClientIPsecParameters',
            sa_life_time_seconds=sa_life_time_seconds,
            sa_data_size_kilobytes=sa_data_size_kilobytes,
            ipsec_encryption=ipsec_encryption,
            ipsec_integrity=ipsec_integrity,
            ike_encryption=ike_encryption,
            ike_integrity=ike_integrity,
            dh_group=dh_group,
            pfs_group=pfs_group
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        params = netconn.virtual_network_gateways.set_vpnclient_ipsec_parameters(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name,
            vpnclient_ipsec_params=paramsmodel,
            **kwargs
        )

        params_result = params.result()
        result = params_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get_vpnclient_ipsec_parameters(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets information about the vpnclient ipsec policy for P2S client of virtual network
        gateway in the specified resource group through Network resource provider.

    :param name: The virtual network gateway name.

    :param resource_group: The name of the resource group.

    CLI Example:
    .. code-block:: bash

        azurerm.network.virtual_network_gateway.get_vpnclient_ipsec_parameters test_name test_group

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)
    try:
        policy = netconn.virtual_network_gateways.get_vpnclient_ipsec_parameters(
            resource_group_name=resource_group,
            virtual_network_gateway_name=name
        )

        policy_result = policy.result()
        result = policy_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def vpn_device_configuration_script(hub, name, resource_group, vendor, device_family, firmware_version, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Gets a xml format representation for vpn device configuration script.

    :param name: The name of the virtual network gateway connection for which the configuration
        script is generated.

    :param resource_group: The name of the resource group.

    :param vendor: The vendor for the vpn device.

    :param device_family: The device family for the vpn device.

    :param firmware_version: The firmware version for the vpn device.

    CLI Example:
    .. code-block:: bash

        azurerm.network.virtual_network_gateway.vpn_device_configuration_script test_name test_group \
                  test_vendor test_device_fam test_version

    '''
    netconn = await hub.exec.utils.azurerm.get_client('network', **kwargs)

    try:
        scriptmodel = await hub.exec.utils.azurerm.create_object_model(
            'network',
            'VpnDeviceScriptParameters',
            vendor=vendor,
            device_family=device_family,
            firmware_version=firmware_version,
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        script = netconn.virtual_network_gateways.vpn_device_configuration_script(
            resource_group_name=resource_group,
            virtual_network_gateway_connection_name=name,
            parameters=scriptmodel,
            **kwargs
        )

        result = script
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('network', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Compute Image Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.compute.models  # pylint: disable=unused-import
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    from msrestazure.tools import is_valid_resource_id
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def create_or_update(hub, name, resource_group, source_vm=None, source_vm_group=None, os_disk=None,
                           data_disks=None, zone_resilient=False,  **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update an image.

    :param name: The image to create.

    :param resource_group: The resource group name assigned to the image.

    :param source_vm: The name of the virtual machine from which the image is created. This parameter
        or a valid os_disk is required.

    :param source_vm_group: The name of the resource group containing the source virtual machine.
        This defaults to the same resource group specified for the resultant image.

    :param os_disk: The resource ID of an operating system disk to use for the image.

    :param data_disks: The resource ID or list of resource IDs associated with data disks to add to
        the image.

    :param zone_resilient: Specifies whether an image is zone resilient or not. Zone resilient images
        can be created only in regions that provide Zone Redundant Storage (ZRS).

    CLI Example:

    .. code-block:: bash

        azurerm.compute.image.create_or_update testimage testgroup

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)

    if source_vm:
        # Use VM name to link to the IDs of existing VMs.
        vm_instance = await hub.exec.azurerm.compute.virtual_machine.get(
            name=source_vm,
            resource_group=(source_vm_group or resource_group),
            log_level='info',
            **kwargs
        )

        if 'error' in vm_instance:
            errmsg = 'The source virtual machine could not be found.'
            log.error(errmsg)
            result = {'error': errmsg}
            return result

        source_vm = {'id': str(vm_instance['id'])}

    spmodel = None
    if os_disk:
        if is_valid_resource_id(os_disk):
            os_disk = {'id': os_disk}
        else:
            errmsg = 'The os_disk parameter is not a valid resource ID string.'
            log.error(errmsg)
            result = {'error': errmsg}
            return result

        if data_disks:
            if isinstance(data_disks, list):
                data_disks = [ {'id': dd} for dd in data_disks ]
            elif isinstance(data_disks, six.string_types):
                data_disks = [ {'id': data_disks} ]
            else:
                errmsg = 'The data_disk parameter is a single resource ID string or a list of resource IDs.'
                log.error(errmsg)
                result = {'error': errmsg}
                return result

        try:
            spmodel = await hub.exec.utils.azurerm.create_object_model(
                'compute',
                'ImageStorageProfile',
                os_disk=os_disk,
                data_disks=data_disks,
                zone_resilient=zone_resilient,
                **kwargs
            )
        except TypeError as exc:
            result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
            return result

    try:
        imagemodel = await hub.exec.utils.azurerm.create_object_model(
            'compute',
            'Image',
            source_virtual_machine=source_vm,
            storage_profile=spmodel,
            **kwargs
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        image = compconn.images.create_or_update(
            resource_group_name=resource_group,
            image_name=name,
            parameters=imagemodel
        )
        image.wait()
        image_result = image.result()
        result = image_result.as_dict()

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete an image.

    :param name: The image to delete.

    :param resource_group: The resource group name assigned to the image.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.image.delete testimage testgroup

    '''
    result = False
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        compconn.images.delete(
            resource_group_name=resource_group,
            image_name=name
        )
        result = True

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Get a dictionary representing an image's properties.

    :param name: The image to get.

    :param resource_group: The resource group name assigned to the image.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.image.get testimage testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        image = compconn.images.get(
            resource_group_name=resource_group,
            image_name=name
        )
        result = image.as_dict()

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def images_list_by_resource_group(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all images within a resource group.

    :param resource_group: The resource group name to list images within.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.image.list_by_resource_group testgroup

    '''
    result = {}
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        images = await hub.exec.utils.azurerm.paged_object_to_list(
            compconn.images.list_by_resource_group(
                resource_group_name=resource_group
            )
        )

        for image in images:
            result[image['name']] = image
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def images_list(hub, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all images in a subscription.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.image.list

    '''
    result = {}
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        images = await hub.exec.utils.azurerm.paged_object_to_list(compconn.images.list())

        for image in images:
            result[image['name']] = image
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Network Public IP Address State Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as a dictionary of
    keyword arguments to the ``connection_auth`` parameter in order to work properly. Since the authentication
    parameters are sensitive, it's recommended to pass them to the states via pillar.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

    **cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud. Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

    Example Pillar for Azure Resource Manager authentication:

    .. code-block:: yaml

        azurerm:
            user_pass_auth:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                username: fletch
                password: 123pass
            mysubscription:
                subscription_id: 3287abc8-f98a-c678-3bde-326766fd3617
                tenant: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                client_id: ABCDEFAB-1234-ABCD-1234-ABCDEFABCDEF
                secret: XXXXXXXXXXXXXXXXXXXXXXXX
                cloud_environment: AZURE_PUBLIC_CLOUD

    Example states using Azure Resource Manager authentication:

    .. code-block:: jinja

        Ensure virtual network exists:
            azurerm.network.virtual_network.present:
                - name: my_vnet
                - resource_group: my_rg
                - address_prefixes:
                    - '10.0.0.0/8'
                    - '192.168.0.0/16'
                - dns_servers:
                    - '8.8.8.8'
                - tags:
                    how_awesome: very
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}

        Ensure virtual network is absent:
            azurerm.network.virtual_network.absent:
                - name: other_vnet
                - resource_group: my_rg
                - connection_auth: {{ profile }}

'''

# Python libs
from __future__ import absolute_import
import logging
import re

try:
    from six.moves import range as six_range
except ImportError:
    six_range = range

log = logging.getLogger(__name__)


async def present(hub, ctx, name, resource_group, tags=None, sku=None, public_ip_allocation_method=None,
            public_ip_address_version=None, dns_settings=None, idle_timeout_in_minutes=None, connection_auth=None,
            **kwargs):
    '''
    .. versionadded:: 1.0.0

    Ensure a public IP address exists.

    :param name:
        Name of the public IP address.

    :param resource_group:
        The resource group assigned to the public IP address.

    :param dns_settings:
        An optional dictionary representing a valid PublicIPAddressDnsSettings object. Parameters include
        'domain_name_label' and 'reverse_fqdn', which accept strings. The 'domain_name_label' parameter is concatenated
        with the regionalized DNS zone make up the fully qualified domain name associated with the public IP address.
        If a domain name label is specified, an A DNS record is created for the public IP in the Microsoft Azure DNS
        system. The 'reverse_fqdn' parameter is a user-visible, fully qualified domain name that resolves to this public
        IP address. If the reverse FQDN is specified, then a PTR DNS record is created pointing from the IP address in
        the in-addr.arpa domain to the reverse FQDN.

    :param sku:
        The public IP address SKU, which can be 'Basic' or 'Standard'.

    :param public_ip_allocation_method:
        The public IP allocation method. Possible values are: 'Static' and 'Dynamic'.

    :param public_ip_address_version:
        The public IP address version. Possible values are: 'IPv4' and 'IPv6'.

    :param idle_timeout_in_minutes:
        An integer representing the idle timeout of the public IP address.

    :param tags:
        A dictionary of strings can be passed as tag metadata to the public IP address object.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.

    Example usage:

    .. code-block:: yaml

        Ensure public IP exists:
            azurerm.network.public_ip_address.present:
                - name: pub_ip1
                - resource_group: group1
                - dns_settings:
                    domain_name_label: decisionlab-ext-test-label
                - sku: basic
                - public_ip_allocation_method: static
                - public_ip_address_version: ipv4
                - idle_timeout_in_minutes: 4
                - tags:
                    contact_name: Elmer Fudd Gantry
                - connection_auth: {{ profile }}
                - require:
                  - azurearm_resource: Ensure resource group exists

    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    if sku:
        sku = {'name': sku.capitalize()}

    pub_ip = await hub.exec.azurerm.network.public_ip_address.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' not in pub_ip:
        # tag changes
        tag_changes = await hub.exec.utils.dictdiffer.deep_diff(pub_ip.get('tags', {}), tags or {})
        if tag_changes:
            ret['changes']['tags'] = tag_changes

        # dns_settings changes
        if dns_settings:
            if not isinstance(dns_settings, dict):
                ret['comment'] = 'DNS settings must be provided as a dictionary!'
                return ret

            for key in dns_settings:
                if dns_settings[key] != pub_ip.get('dns_settings', {}).get(key):
                    ret['changes']['dns_settings'] = {
                        'old': pub_ip.get('dns_settings'),
                        'new': dns_settings
                    }
                    break

        # sku changes
        if sku:
            sku_changes = await hub.exec.utils.dictdiffer.deep_diff(pub_ip.get('sku', {}), sku)
            if sku_changes:
                ret['changes']['sku'] = sku_changes

        # public_ip_allocation_method changes
        if public_ip_allocation_method:
            if public_ip_allocation_method.capitalize() != pub_ip.get('public_ip_allocation_method'):
                ret['changes']['public_ip_allocation_method'] = {
                    'old': pub_ip.get('public_ip_allocation_method'),
                    'new': public_ip_allocation_method
                }

        # public_ip_address_version changes
        if public_ip_address_version:
            if public_ip_address_version.lower() != pub_ip.get('public_ip_address_version', '').lower():
                ret['changes']['public_ip_address_version'] = {
                    'old': pub_ip.get('public_ip_address_version'),
                    'new': public_ip_address_version
                }

        # idle_timeout_in_minutes changes
        if idle_timeout_in_minutes and (int(idle_timeout_in_minutes) != pub_ip.get('idle_timeout_in_minutes')):
            ret['changes']['idle_timeout_in_minutes'] = {
                'old': pub_ip.get('idle_timeout_in_minutes'),
                'new': idle_timeout_in_minutes
            }

        if not ret['changes']:
            ret['result'] = True
            ret['comment'] = 'Public IP address {0} is already present.'.format(name)
            return ret

        if ctx['test']:
            ret['result'] = None
            ret['comment'] = 'Public IP address {0} would be updated.'.format(name)
            return ret

    else:
        ret['changes'] = {
            'old': {},
            'new': {
                'name': name,
                'tags': tags,
                'dns_settings': dns_settings,
                'sku': sku,
                'public_ip_allocation_method': public_ip_allocation_method,
                'public_ip_address_version': public_ip_address_version,
                'idle_timeout_in_minutes': idle_timeout_in_minutes,
            }
        }

    if ctx['test']:
        ret['comment'] = 'Public IP address {0} would be created.'.format(name)
        ret['result'] = None
        return ret

    pub_ip_kwargs = kwargs.copy()
    pub_ip_kwargs.update(connection_auth)

    pub_ip = await hub.exec.azurerm.network.public_ip_address.create_or_update(
        name=name,
        resource_group=resource_group,
        sku=sku,
        tags=tags,
        dns_settings=dns_settings,
        public_ip_allocation_method=public_ip_allocation_method,
        public_ip_address_version=public_ip_address_version,
        idle_timeout_in_minutes=idle_timeout_in_minutes,
        **pub_ip_kwargs
    )

    if 'error' not in pub_ip:
        ret['result'] = True
        ret['comment'] = 'Public IP address {0} has been created.'.format(name)
        return ret

    ret['comment'] = 'Failed to create public IP address {0}! ({1})'.format(name, pub_ip.get('error'))
    return ret


async def absent(hub, ctx, name, resource_group, connection_auth=None):
    '''
    .. versionadded:: 1.0.0

    Ensure a public IP address does not exist in the resource group.

    :param name:
        Name of the public IP address.

    :param resource_group:
        The resource group assigned to the public IP address.

    :param connection_auth:
        A dict with subscription and authentication parameters to be used in connecting to the
        Azure Resource Manager API.
    '''
    ret = {
        'name': name,
        'result': False,
        'comment': '',
        'changes': {}
    }

    if not isinstance(connection_auth, dict):
        ret['comment'] = 'Connection information must be specified via connection_auth dictionary!'
        return ret

    pub_ip = await hub.exec.azurerm.network.public_ip_address.get(
        name,
        resource_group,
        azurearm_log_level='info',
        **connection_auth
    )

    if 'error' in pub_ip:
        ret['result'] = True
        ret['comment'] = 'Public IP address {0} was not found.'.format(name)
        return ret

    elif ctx['test']:
        ret['comment'] = 'Public IP address {0} would be deleted.'.format(name)
        ret['result'] = None
        ret['changes'] = {
            'old': pub_ip,
            'new': {},
        }
        return ret

    deleted = await hub.exec.azurerm.network.public_ip_address.delete(name, resource_group, **connection_auth)

    if deleted:
        ret['result'] = True
        ret['comment'] = 'Public IP address {0} has been deleted.'.format(name)
        ret['changes'] = {
            'old': pub_ip,
            'new': {}
        }
        return ret

    ret['comment'] = 'Failed to delete public IP address {0}!'.format(name)
    return ret



# -*- coding: utf-8 -*-
'''
Azure Resource Manager (ARM) Compute Virtual Machine Execution Module

.. versionadded:: 1.0.0

:maintainer: <devops@eitr.tech>
:maturity: new
:depends:
    * `azure <https://pypi.python.org/pypi/azure>`_ >= 4.0.0
    * `azure-common <https://pypi.python.org/pypi/azure-common>`_ >= 1.1.23
    * `azure-mgmt <https://pypi.python.org/pypi/azure-mgmt>`_ >= 4.0.0
    * `azure-mgmt-compute <https://pypi.python.org/pypi/azure-mgmt-compute>`_ >= 4.6.2
    * `azure-mgmt-network <https://pypi.python.org/pypi/azure-mgmt-network>`_ >= 4.0.0
    * `azure-mgmt-resource <https://pypi.python.org/pypi/azure-mgmt-resource>`_ >= 2.2.0
    * `azure-mgmt-storage <https://pypi.python.org/pypi/azure-mgmt-storage>`_ >= 2.0.0
    * `azure-mgmt-web <https://pypi.python.org/pypi/azure-mgmt-web>`_ >= 0.35.0
    * `azure-storage <https://pypi.python.org/pypi/azure-storage>`_ >= 0.36.0
    * `msrestazure <https://pypi.python.org/pypi/msrestazure>`_ >= 0.6.1
:platform: linux

:configuration: This module requires Azure Resource Manager credentials to be passed as keyword arguments
    to every function in order to work properly.

    Required provider parameters:

    if using username and password:
      * ``subscription_id``
      * ``username``
      * ``password``

    if using a service principal:
      * ``subscription_id``
      * ``tenant``
      * ``client_id``
      * ``secret``

    Optional provider parameters:

**cloud_environment**: Used to point the cloud driver to different API endpoints, such as Azure GovCloud.
    Possible values:
      * ``AZURE_PUBLIC_CLOUD`` (default)
      * ``AZURE_CHINA_CLOUD``
      * ``AZURE_US_GOV_CLOUD``
      * ``AZURE_GERMAN_CLOUD``

'''

# Python libs
from __future__ import absolute_import
import logging
import os

# Azure libs
HAS_LIBS = False
try:
    import azure.mgmt.compute.models  # pylint: disable=unused-import
    from msrest.exceptions import SerializationError
    from msrestazure.azure_exceptions import CloudError
    from msrestazure.tools import is_valid_resource_id, parse_resource_id
    HAS_LIBS = True
except ImportError:
    pass

log = logging.getLogger(__name__)


async def create_or_update(hub, name, resource_group, vm_size, admin_username='idem', os_disk_create_option='FromImage',
                           os_disk_size_gb=30, ssh_public_keys=None, allocate_public_ip=False,
                           create_interfaces=True, network_resource_group=None, virtual_network=None,
                           subnet=None, network_interfaces=None, os_disk_vhd_uri=None, os_disk_image_uri=None,
                           os_type=None, os_disk_name=None, os_disk_caching=None, image=None, admin_password=None,
                           **kwargs):
    '''
    .. versionadded:: 1.0.0

    Create or update a virtual machine.

    :param name: The virtual machine to create.

    :param resource_group: The resource group name assigned to the virtual machine.

    :param vm_size: The size of the virtual machine.

    # These can be passed as kwargs:
    #   priority = low or regular
    #   eviction_policy = deallocate or delete
    #   license_type = Windows_Client or Windows_Server
    #   zones = [ list of zone numbers ]

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.create_or_update testvm testgroup

    '''
    if 'location' not in kwargs:
        rg_props = await hub.exec.azurerm.resource.group.get(
            resource_group, **kwargs
        )

        if 'error' in rg_props:
            log.error(
                'Unable to determine location from resource group specified.'
            )
            return False
        kwargs['location'] = rg_props['location']

    if not network_interfaces:
        network_interfaces = []

    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)

    params = kwargs.copy()

    # This section creates dictionaries if required in order to properly create SubResource objects
    if 'availability_set' in params and not isinstance(params['availability_set'], dict):
        params.update({'availability_set': {'id': params['availability_set']}})

    if 'virtual_machine_scale_set' in params and not isinstance(params['virtual_machine_scale_set'], dict):
        params.update({'virtual_machine_scale_set': {'id': params['virtual_machine_scale_set']}})

    if 'proximity_placement_group' in params and not isinstance(params['proximity_placement_group'], dict):
        params.update({'proximity_placement_group': {'id': params['proximity_placement_group']}})

    if 'host' in params and not isinstance(params['host'], dict):
        params.update({'host': {'id': params['host']}})

    if os_disk_image_uri and not isinstance(os_disk_image_uri, dict):
        os_disk_image_uri = {'id': os_disk_image_uri}

    if os_disk_vhd_uri and not isinstance(os_disk_vhd_uri, dict):
        os_disk_vhd_uri = {'id': os_disk_vhd_uri}

    if not network_interfaces and create_interfaces:
        ipc = {'name': f'{name}-iface0-ip'}

        if allocate_public_ip:
            pubip = await hub.exec.azurerm.network.public_ip_address.create_or_update(
                f'{name}-ip',
                resource_group,
                **kwargs
            )

            try:
                ipc.update({'public_ip_address': {'id': pubip['id']}})
            except KeyError as exc:
                result = {'error': 'The public IP address could not be created. ({0})'.format(str(exc))}
                return result

        iface = await hub.exec.azurerm.network.network_interface.create_or_update(
            f'{name}-iface0',
            [ipc],
            subnet,
            virtual_network,
            network_resource_group or resource_group,
            **kwargs
        )

        try:
            nic = {'id': iface['id']}
        except KeyError as exc:
            result = {'error': 'The network interface could not be created. ({0})'.format(str(exc))}
            return result

        network_interfaces.append(nic)

    params.update(
        {
            #'plan': {
            #    'name' None,
            #    'publisher': None,
            #    'product': None,
            #    'promotion_code': None
            #},
            'hardware_profile': {
                'vm_size': vm_size.lower()
            },
            'storage_profile': {
                'os_disk': {
                    'os_type': os_type,
                    #'encryption_settings': {
                    #    'disk_encryption_key': {
                    #        'secret_url': '',
                    #        'source_vault': { id: '' }
                    #    },
                    #    'key_encryption_key': {
                    #        'key_url': '',
                    #        'source_vault': { id: '' }
                    #    },
                    #    'enabled': None # True or False
                    #},
                    'name': os_disk_name,
                    'vhd': os_disk_vhd_uri,
                    'image': os_disk_image_uri,
                    'caching': os_disk_caching, # ReadOnly or ReadWrite
                    #'write_accelerator_enabled': None, # True or False
                    #'diff_disk_settings': { 'option': None }, # Local or None
                    'create_option': os_disk_create_option, # Attach or FromImage
                    'disk_size_gb': os_disk_size_gb,
                    #'managed_disk': { 'id': None, 'storage_account_type': None } # (Standard|Premium)_LRS or (Standard|Ultra)SSD_LRS
                },
                'data_disks': [
                    #{
                    #    'lun': None,
                    #    'name': None,
                    #    'vhd': { 'uri': '' },
                    #    'image': { 'uri': '' },
                    #    'caching': None, # ReadOnly or ReadWrite
                    #    'write_accelerator_enabled': None, # True or False
                    #    'create_option': None, # Attach or FromImage
                    #    'disk_size_gb': None,
                    #    'managed_disk': { 'id': None, 'storage_account_type': None }, # (Standard|Premium)_LRS or (Standard|Ultra)SSD_LRS
                    #    'to_be_detached': None # True or False
                    #}
                ]
            },
            #'additional_capabilities': {
            #    'ultra_ssd_enabled': None
            #},
            'os_profile': {
                'computer_name': name,
                'admin_username': admin_username,
                'admin_password': admin_password,
            #    'custom_data': None,
            #    'windows_configuration': None,
            #    'secrets': None,
            #    'allow_extension_operations': None,
            #    'require_guest_provision_signal': None
            },
            'network_profile': {
                'network_interfaces': network_interfaces
            },
            #'diagnostics_profiles': {
            #    'boot_diagnostics': {
            #        'enabled': None, # True or False
            #        'storage_uri': # storage account URI
            #    }
            #},
            #'identity': {
            #    'type': None, # SystemAssigned or UserAssigned
            #    'user_assigned_identities': None # VirtualMachineIdentityUserAssignedIdentitiesValue
            #},
            #'billing_profile': { max_price: None }
        }
    )

    if isinstance(ssh_public_keys, list):
        pubkeys = []
        for pubkey in ssh_public_keys:
            if os.path.isfile(pubkey):
                try:
                    with open(pubkey, 'r') as pubkey_file:
                        pubkeys.append(
                            {
                                'key_data': pubkey_file.read(),
                                'path': f'/home/{admin_username}/.ssh/authorized_keys'
                            }
                        )
                except FileNotFoundError as exc:
                    log.error(
                        'Unable to open ssh public key file: %s (%s)', pubkey, exc
                    )
            else:
                pubkeys.append(
                    {
                        'key_data': pubkey,
                        'path': f'/home/{admin_username}/.ssh/authorized_keys'
                    }
                )

        params['os_profile'].update(
            {
                'linux_configuration': {
                    'disable_password_authentication': True,
                    'ssh': {
                        'public_keys': pubkeys
                    }
                }
            }
        )

    if image:
        if is_valid_resource_id(image):
            params['storage_profile'].update(
                { 'image_reference': { 'id': image }}
            )
        elif '|' in image:
            image_keys = ['publisher', 'offer', 'sku', 'version']
            params['storage_profile'].update(
                { 'image_reference': dict(zip(image_keys, image.split('|'))) }
            )

    try:
        vmmodel = await hub.exec.utils.azurerm.create_object_model(
            'compute',
            'VirtualMachine',
            **params
        )
    except TypeError as exc:
        result = {'error': 'The object model could not be built. ({0})'.format(str(exc))}
        return result

    try:
        vm = compconn.virtual_machines.create_or_update(
            resource_group_name=resource_group,
            vm_name=name,
            parameters=vmmodel
        )

        vm.wait()
        vm_result = vm.result()
        result = vm_result.as_dict()

        network_interfaces = []

        # Give some more details about the sub-objects
        for iface in result['network_profile']['network_interfaces']:
            iface_dict = parse_resource_id(
                iface['id']
            )

            iface_details = await hub.exec.azurerm.network.network_interface.get(
                resource_group=iface_dict['resource_group'],
                name=iface_dict['name'],
                **kwargs
            )

            network_interfaces.append(iface_details)

        result['network_profile']['network_interfaces'] = network_interfaces

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}
    except SerializationError as exc:
        result = {'error': 'The object model could not be parsed. ({0})'.format(str(exc))}

    return result


async def delete(hub, name, resource_group, cleanup_disks=False, cleanup_data_disks=False, cleanup_interfaces=False,
                 **kwargs):
    '''
    .. versionadded:: 1.0.0

    Delete a virtual machine.

    :param name: The virtual machine to delete.

    :param resource_group: The resource group name assigned to the virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.delete testvm testgroup

    '''
    result = False
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)

    vm = await hub.exec.azurerm.compute.virtual_machine.get(
        resource_group=resource_group,
        name=name,
        **kwargs
    )

    try:
        poller = compconn.virtual_machines.delete(
            resource_group_name=resource_group,
            vm_name=name
        )

        poller.wait()

        if cleanup_disks:
            os_disk = parse_resource_id(
                vm['storage_profile']['os_disk'].get('managed_disk', {}).get('id')
            )

            os_disk_ret = await hub.exec.azurerm.compute.disk.delete(
                resource_group=os_disk['resource_group'],
                name=os_disk['name'],
                **kwargs
            )

        if cleanup_data_disks:
            for disk in vm['storage_profile']['data_disks']:
                disk_dict = parse_resource_id(
                    disk.get('managed_disk', {}).get('id')
                )

                data_disk_ret = await hub.exec.azurerm.compute.disk.delete(
                    resource_group=disk_dict['resource_group'],
                    name=disk_dict['name'],
                    **kwargs
                )

        if cleanup_interfaces:
            for iface in vm['network_profile']['network_interfaces']:
                iface_dict = parse_resource_id(
                    iface['id']
                )

                iface_details = await hub.exec.azurerm.network.network_interface.get(
                    resource_group=iface_dict['resource_group'],
                    name=iface_dict['name'],
                    **kwargs
                )

                iface_ret = await hub.exec.azurerm.network.network_interface.delete(
                    resource_group=iface_dict['resource_group'],
                    name=iface_dict['name'],
                    **kwargs
                )

                for ipc in iface_details['ip_configurations']:
                    if ipc.get('public_ip_address'):
                        ip_dict = parse_resource_id(
                            ipc['public_ip_address']['id']
                        )

                        ip_ret = await hub.exec.azurerm.network.public_ip_address.delete(
                            resource_group=ip_dict['resource_group'],
                            name=ip_dict['name'],
                            **kwargs
                        )

        result = True

    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)

    return result


async def capture(hub, name, destination_name, resource_group, prefix='capture-', overwrite=False, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Captures the VM by copying virtual hard disks of the VM and outputs
    a template that can be used to create similar VMs.

    :param name: The name of the virtual machine.

    :param destination_name: The destination container name.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    :param prefix: (Default: 'capture-') The captured virtual hard disk's name prefix.

    :param overwrite: (Default: False) Overwrite the destination disk in case of conflict.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.capture testvm testcontainer testgroup

    '''
    # pylint: disable=invalid-name
    VirtualMachineCaptureParameters = getattr(
        azure.mgmt.compute.models, 'VirtualMachineCaptureParameters'
    )

    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.capture(
            resource_group_name=resource_group,
            vm_name=name,
            parameters=VirtualMachineCaptureParameters(
                vhd_prefix=prefix,
                destination_container_name=destination_name,
                overwrite_vhds=overwrite
            )
        )
        vm.wait()
        vm_result = vm.result()
        result = vm_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def get(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Retrieves information about the model view or the instance view of a
    virtual machine.

    :param name: The name of the virtual machine.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.get testvm testgroup

    '''
    expand = kwargs.get('expand')

    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.get(
            resource_group_name=resource_group,
            vm_name=name,
            expand=expand
        )
        result = vm.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def convert_to_managed_disks(hub, name, resource_group, **kwargs):  # pylint: disable=invalid-name
    '''
    .. versionadded:: 1.0.0

    Converts virtual machine disks from blob-based to managed disks. Virtual
    machine must be stop-deallocated before invoking this operation.

    :param name: The name of the virtual machine to convert.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.convert_to_managed_disks testvm testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.convert_to_managed_disks(
            resource_group_name=resource_group,
            vm_name=name
        )
        vm.wait()
        vm_result = vm.result()
        result = vm_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def deallocate(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Power off a virtual machine and deallocate compute resources.

    :param name: The name of the virtual machine to deallocate.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.deallocate testvm testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    result = False
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.deallocate(
            resource_group_name=resource_group,
            vm_name=name
        )
        vm.wait()
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def generalize(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Set the state of a virtual machine to 'generalized'.

    :param name: The name of the virtual machine.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.generalize testvm testgroup

    '''
    result = False
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        compconn.virtual_machines.generalize(
            resource_group_name=resource_group,
            vm_name=name
        )
        result = True
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)

    return result


async def list_(hub, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all virtual machines within a resource group.

    :param resource_group: The resource group name to list virtual
        machines within.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.list testgroup

    '''
    result = {}
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        vms = await hub.exec.utils.azurerm.paged_object_to_list(
            compconn.virtual_machines.list(
                resource_group_name=resource_group
            )
        )
        for vm in vms:  # pylint: disable=invalid-name
            result[vm['name']] = vm
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_all(hub, **kwargs):
    '''
    .. versionadded:: 1.0.0

    List all virtual machines within a subscription.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.list_all

    '''
    result = {}
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        vms = await hub.exec.utils.azurerm.paged_object_to_list(
            compconn.virtual_machines.list_all()
        )
        for vm in vms:  # pylint: disable=invalid-name
            result[vm['name']] = vm
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def list_available_sizes(hub, name, resource_group, **kwargs):  # pylint: disable=invalid-name
    '''
    .. versionadded:: 1.0.0

    Lists all available virtual machine sizes to which the specified virtual
    machine can be resized.

    :param name: The name of the virtual machine.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.list_available_sizes testvm testgroup

    '''
    result = {}
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        sizes = await hub.exec.utils.azurerm.paged_object_to_list(
            compconn.virtual_machines.list_available_sizes(
                resource_group_name=resource_group,
                vm_name=name
            )
        )
        for size in sizes:
            result[size['name']] = size
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def power_off(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Power off (stop) a virtual machine.

    :param name: The name of the virtual machine to stop.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.power_off testvm testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.power_off(
            resource_group_name=resource_group,
            vm_name=name
        )
        vm.wait()
        vm_result = vm.result()
        result = vm_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def restart(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Restart a virtual machine.

    :param name: The name of the virtual machine to restart.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.restart testvm testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.restart(
            resource_group_name=resource_group,
            vm_name=name
        )
        vm.wait()
        vm_result = vm.result()
        result = vm_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def start(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Power on (start) a virtual machine.

    :param name: The name of the virtual machine to start.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.start testvm testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.start(
            resource_group_name=resource_group,
            vm_name=name
        )
        vm.wait()
        vm_result = vm.result()
        result = vm_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result


async def redeploy(hub, name, resource_group, **kwargs):
    '''
    .. versionadded:: 1.0.0

    Redeploy a virtual machine.

    :param name: The name of the virtual machine to redeploy.

    :param resource_group: The resource group name assigned to the
        virtual machine.

    CLI Example:

    .. code-block:: bash

        azurerm.compute.virtual_machine.redeploy testvm testgroup

    '''
    compconn = await hub.exec.utils.azurerm.get_client('compute', **kwargs)
    try:
        # pylint: disable=invalid-name
        vm = compconn.virtual_machines.redeploy(
            resource_group_name=resource_group,
            vm_name=name
        )
        vm.wait()
        vm_result = vm.result()
        result = vm_result.as_dict()
    except CloudError as exc:
        await hub.exec.utils.azurerm.log_cloud_error('compute', str(exc), **kwargs)
        result = {'error': str(exc)}

    return result
